//
// Created: 1/31/03, Klaus Salchner
// created this basic authentication and form framework; added buttons to check for 
// security roles

// Updated: 2/1/03, Klaus Salchner
// Added buttons to check for each security permission and to show the user info;
// added logic for closing the window and log off
//
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Security.Permissions;
using System.Security;
using System.Threading;
using System.Security.Principal;
using System.Globalization;

namespace RetrieveSecurity
{
	/// <summary>
	/// sample application which demonstrates how to authenticate a user against a table in the
	/// database and then associate the secuirty information for this user with the ISecurityPrincipal
	/// of the thread so we can check security in a declarative manner leveraging the .NET security
	/// framework and classes
	/// </summary>
	public class RetrieveSecurity : System.Windows.Forms.Form
	{
		private const string MessageCaption = "MissingInformation";
		private const string NoValidUserNameOrPassword = "NoValidUserNameOrPassword";
		private const string WrongUserNameAndPassword = "WrongUserNameAndPassword";
		private const string SecurityGroupListName = "SecurityGroupList";
		private const string SecurityRightListName = "SecurityRightList";
		private const string UserInfoListName = "UserInfoList";
		private const string UserRole = "User";
		private const string PowerUserRole = "PowerUser";
		private const string AdministratorRole = "Administrator";
		private const string SuperAdministratorRole = "SuperAdministrator";
		private const string RoleOperationCompleted = "RoleOperationCompleted";
		private const string DoNotHaveRole = "DoNotHaveRole";
		private const string Cancel = "CancelButton";
		private const string UserDetails = "UserDetails";
		private const string StandardUserInfo = "StandardUserInfo";
		private const string CustomUserInfo = "CustomUserInfo";
		private const string StandardPropertyName = "StandardPropertyName";
		private const string ViewBudget = "ViewBudget";
		private const string ViewInventory = "ViewInventory";
		private const string ViewRevenue = "ViewRevenue";
		private const string ViewProfit = "ViewProfit";
		private const string ModifyBudget = "ModifyBudget";
		private const string ModifyInventory = "ModifyInventory";
		private const string ModifyRevenue = "ModifyRevenue";
		private const string ModifyProfit = "ModifyProfit";
		private const string HasNotPermission = "HasNotPermission";
		private const string HasPermission = "HasPermission";
		private const string LogOff = "LogOff";
		private IPrincipal OriginalPrincipal;
		private System.Windows.Forms.TextBox textUserName;
		private System.Windows.Forms.TextBox textPassword;
		private System.Windows.Forms.Label labelUserName;
		private System.Windows.Forms.Button btnLogOn;
		private System.Windows.Forms.Label labelPassword;

		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		/// <summary>
		/// initialize the windows form which shows at the beginning a logon form; after the 
		/// user has been logged on we dynamically change the form to show how the .NET
		/// security framework works
		/// </summary>
		public RetrieveSecurity()
		{
			InitializeComponent();
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.textUserName = new System.Windows.Forms.TextBox();
			this.textPassword = new System.Windows.Forms.TextBox();
			this.labelUserName = new System.Windows.Forms.Label();
			this.labelPassword = new System.Windows.Forms.Label();
			this.btnLogOn = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// textUserName
			// 
			this.textUserName.Location = new System.Drawing.Point(80, 16);
			this.textUserName.Name = "textUserName";
			this.textUserName.Size = new System.Drawing.Size(160, 20);
			this.textUserName.TabIndex = 0;
			this.textUserName.Text = "";
			// 
			// textPassword
			// 
			this.textPassword.Location = new System.Drawing.Point(80, 48);
			this.textPassword.Name = "textPassword";
			this.textPassword.PasswordChar = '*';
			this.textPassword.Size = new System.Drawing.Size(160, 20);
			this.textPassword.TabIndex = 1;
			this.textPassword.Text = "";
			// 
			// labelUserName
			// 
			this.labelUserName.Location = new System.Drawing.Point(8, 16);
			this.labelUserName.Name = "labelUserName";
			this.labelUserName.Size = new System.Drawing.Size(64, 23);
			this.labelUserName.TabIndex = 2;
			this.labelUserName.Text = "&Username";
			// 
			// labelPassword
			// 
			this.labelPassword.Location = new System.Drawing.Point(8, 48);
			this.labelPassword.Name = "labelPassword";
			this.labelPassword.Size = new System.Drawing.Size(56, 23);
			this.labelPassword.TabIndex = 3;
			this.labelPassword.Text = "&Password";
			// 
			// btnLogOn
			// 
			this.btnLogOn.Location = new System.Drawing.Point(80, 88);
			this.btnLogOn.Name = "btnLogOn";
			this.btnLogOn.Size = new System.Drawing.Size(74, 23);
			this.btnLogOn.TabIndex = 4;
			this.btnLogOn.Text = "&Log on";
			this.btnLogOn.Click += new System.EventHandler(this.btnLogOn_Click);
			// 
			// RetrieveSecurity
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(256, 125);
			this.Controls.Add(this.btnLogOn);
			this.Controls.Add(this.labelPassword);
			this.Controls.Add(this.labelUserName);
			this.Controls.Add(this.textPassword);
			this.Controls.Add(this.textUserName);
			this.Name = "RetrieveSecurity";
			this.Text = "Retrieve security";
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// the main entry point for the application; we show the form which shows at the 
		/// beginning the logon to the user
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new RetrieveSecurity());
		}

		/// <summary>
		/// the user has entered the username/pwd and we check it against the database
		/// </summary>
		/// <param name="sender">the object which sends the event</param>
		/// <param name="e">the event handler args</param>
		private void btnLogOn_Click(object sender, System.EventArgs e)
		{
			int UserId;

			// check that we have a valid user name and password entered
			if ((textUserName.Text.Length <= 0) || (textPassword.Text.Length <=0))
			{
				MessageBox.Show(this,DataLayer.GetString(NoValidUserNameOrPassword),DataLayer.GetString(MessageCaption));

				// clear any password which has been entered
				textPassword.Text = String.Empty;

				return;
			}

			try
			{
				// show warning that the username & pwd does not exist
				if (!DataLayer.CheckUserNameAndPassword(textUserName.Text,textPassword.Text,out UserId))
				{
					MessageBox.Show(this,DataLayer.GetString(WrongUserNameAndPassword),DataLayer.GetString(MessageCaption));

					// set the username and password empty
					textUserName.Text = String.Empty;
					textPassword.Text = String.Empty;

					return;
				}

				// retrieve all the security groups and security rights the user belongs too
				Hashtable SecurityGroups;
				Hashtable SecurityRights;
				DataLayer.RetrieveSecurityInformation(UserId,out SecurityGroups,out SecurityRights);

				// retrieve information of the user itself
				Hashtable UserInfo;
				DataLayer.RetrieveUserInformation(UserId,out UserInfo);

				// creates a new IPrincipal and IIdentity object and sets them for this thread and
				// all future threads of the app-domain; we get the current IPrincipal returned so
				// we can hold on to it and restore it later on
				OriginalPrincipal = SecurityPrincipal.SetSecurityPrincipal(SecurityGroups,SecurityRights,UserInfo);

				// hide all existing controls
				labelUserName.Visible = false;
				labelPassword.Visible = false;
				textUserName.Visible = false;
				textPassword.Visible = false;
				btnLogOn.Visible = false;

				// clears the username and password of the logged on user
				textUserName.Text = String.Empty;
				textPassword.Text = String.Empty;

				// add the two list box with the list of security groups and security rights
				AddListBoxWithValues(new Point(20,20 ),new Size(100,100),SecurityGroupListName,SecurityGroups);
				AddListBoxWithValues(new Point(140,20),new Size(100,100),SecurityRightListName,SecurityRights);
				AddListBoxWithValues(new Point(260,20),new Size(160,100),UserInfoListName,UserInfo);

				// for each security group/role we want to demenostrate we add a button
				AddButtonWithEventHandler(new Point(50,140),UserRole,UserRole,new EventHandler(User_Click));
				AddButtonWithEventHandler(new Point(50,180),PowerUserRole,PowerUserRole,new EventHandler(PowerUser_Click));
				AddButtonWithEventHandler(new Point(50,220),AdministratorRole,AdministratorRole,new EventHandler(Administrator_Click));
				AddButtonWithEventHandler(new Point(50,260),SuperAdministratorRole,SuperAdministratorRole,new EventHandler(SuperAdministrator_Click));

				// buttons to check whether the user has a specific secuirty right or not
				AddButtonWithEventHandler(new Point(170,140),ViewBudget,ViewBudget,new EventHandler(ViewBudget_Click));
				AddButtonWithEventHandler(new Point(170,180),ViewInventory,ViewInventory,new EventHandler(ViewInventory_Click));
				AddButtonWithEventHandler(new Point(170,220),ViewRevenue,ViewRevenue,new EventHandler(ViewRevenue_Click));
				AddButtonWithEventHandler(new Point(170,260),ViewProfit,ViewProfit,new EventHandler(ViewProfit_Click));

				AddButtonWithEventHandler(new Point(290,140),ModifyBudget,ModifyBudget,new EventHandler(ModifyBudget_Click));
				AddButtonWithEventHandler(new Point(290,180),ModifyInventory,ModifyInventory,new EventHandler(ModifyInventory_Click));
				AddButtonWithEventHandler(new Point(290,220),ModifyRevenue,ModifyRevenue,new EventHandler(ModifyRevenue_Click));
				AddButtonWithEventHandler(new Point(290,260),ModifyProfit,ModifyProfit,new EventHandler(ModifyProfit_Click));

				// add a cancel button 
				this.CancelButton = AddButtonWithEventHandler(new Point(50,300),Cancel,Cancel,new EventHandler(Cancel_Click));

				// button to obtain user details through IPrincipal.IIdentiy
				AddButtonWithEventHandler(new Point(170,300),UserDetails,UserDetails,new EventHandler(UserDetails_Click));
				AddButtonWithEventHandler(new Point(290,300),LogOff,LogOff,new EventHandler(LogOff_Click));

				// resize the form itself
				this.Size = new Size(450, 370);
			}

			// catch any exception we through in the DataLayer and show the message to the user
			catch (DataLayerException DataLayerError)
			{
				MessageBox.Show(this,DataLayerError.Message,DataLayer.GetString(MessageCaption));
			}
		}

		/// <summary>
		/// the event handler for the cancel button which closes the form
		/// </summary>
		/// <param name="sender">the object which sends the event</param>
		/// <param name="e">the event handler args</param>
		private void Cancel_Click(object sender,EventArgs e)
		{
			this.Close();
		}

		/// <summary>
		/// the User button event handler
		/// </summary>
		/// <param name="sender">the object which sends the event</param>
		/// <param name="e">the event handler args</param>
		private void User_Click(object sender,EventArgs e)
		{
			CheckSecurityRole(UserRole);
		}

		/// <summary>
		/// the Power User button event handler
		/// </summary>
		/// <param name="sender">the object which sends the event</param>
		/// <param name="e">the event handler args</param>
		private void PowerUser_Click(object sender,EventArgs e)
		{
			CheckSecurityRole(PowerUserRole);
		}

		/// <summary>
		/// the Administrator button event handler
		/// </summary>
		/// <param name="sender">the object which sends the event</param>
		/// <param name="e">the event handler args</param>
		private void Administrator_Click(object sender,EventArgs e)
		{
			CheckSecurityRole(AdministratorRole);
		}

		/// <summary>
		/// the Super Administrator button event handler
		/// </summary>
		/// <param name="sender">the object which sends the event</param>
		/// <param name="e">the event handler args</param>
		private void SuperAdministrator_Click(object sender,EventArgs e)
		{
			CheckSecurityRole(SuperAdministratorRole);
		}

		/// <summary>
		/// demands a security role and then checks whether the user has that role or not (by
		/// ctaching the appropriate exception); then shows a success or failure message
		/// </summary>
		/// <param name="SecurityRole">the security role we want to demand</param>
		private void CheckSecurityRole(string SecurityRole)
		{
			// create a principal permission obejct which we use to demand a certain security role
			PrincipalPermission Permission = new PrincipalPermission(Thread.CurrentPrincipal.Identity.Name,SecurityRole,true);
			
			// demand the security role
			try
			{
				Permission.Demand();
			}

			// if the user is not member of that user role we get this exception which we catch and show
			// to the user that he does not have the right to perfom this action
			catch (SecurityException TheException)
			{
				MessageBox.Show(this,DataLayer.GetString(DoNotHaveRole),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Warning);
				return;
			}

			// we passed the security check and show the user that the action has been performed
			MessageBox.Show(this,DataLayer.GetString(RoleOperationCompleted),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Information);
		}

		/// <summary>
		/// checks whether the user has view budget rights
		/// </summary>
		/// <param name="sender">the object which sends the event</param>
		/// <param name="e">the event handler args</param>
		private void ViewBudget_Click(object sender,EventArgs e)
		{
			if (CheckSecurityPermission(ViewBudget))
				MessageBox.Show(this,DataLayer.GetString(HasPermission),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Information);
			else
				MessageBox.Show(this,DataLayer.GetString(HasNotPermission),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Warning);
		}

		/// <summary>
		/// checks whether the user has view inventory rights
		/// </summary>
		/// <param name="sender">the object which sends the event</param>
		/// <param name="e">the event handler args</param>
		private void ViewInventory_Click(object sender,EventArgs e)
		{
			if (CheckSecurityPermission(ViewInventory))
				MessageBox.Show(this,DataLayer.GetString(HasPermission),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Information);
			else
				MessageBox.Show(this,DataLayer.GetString(HasNotPermission),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Warning);
		}

		/// <summary>
		/// checks whether the user has view revenue rights
		/// </summary>
		/// <param name="sender">the object which sends the event</param>
		/// <param name="e">the event handler args</param>
		private void ViewRevenue_Click(object sender,EventArgs e)
		{
			if (CheckSecurityPermission(ViewRevenue))
				MessageBox.Show(this,DataLayer.GetString(HasPermission),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Information);
			else
				MessageBox.Show(this,DataLayer.GetString(HasNotPermission),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Warning);
		}

		/// <summary>
		/// checks whether the user has view profit rights
		/// </summary>
		/// <param name="sender">the object which sends the event</param>
		/// <param name="e">the event handler args</param>
		private void ViewProfit_Click(object sender,EventArgs e)
		{
			if (CheckSecurityPermission(ViewProfit))
				MessageBox.Show(this,DataLayer.GetString(HasPermission),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Information);
			else
				MessageBox.Show(this,DataLayer.GetString(HasNotPermission),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Warning);
		}
	
		/// <summary>
		/// checks whether the user has modify budget rights
		/// </summary>
		/// <param name="sender">the object which sends the event</param>
		/// <param name="e">the event handler args</param>
		private void ModifyBudget_Click(object sender,EventArgs e)
		{
			if (CheckSecurityPermission(ModifyBudget))
				MessageBox.Show(this,DataLayer.GetString(HasPermission),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Information);
			else
				MessageBox.Show(this,DataLayer.GetString(HasNotPermission),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Warning);
		}

		/// <summary>
		/// checks whether the user has modify inventory rights
		/// </summary>
		/// <param name="sender">the object which sends the event</param>
		/// <param name="e">the event handler args</param>
		private void ModifyInventory_Click(object sender,EventArgs e)
		{
			if (CheckSecurityPermission(ModifyInventory))
				MessageBox.Show(this,DataLayer.GetString(HasPermission),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Information);
			else
				MessageBox.Show(this,DataLayer.GetString(HasNotPermission),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Warning);
		}

		/// <summary>
		/// checks whether the user has modify revenue rights
		/// </summary>
		/// <param name="sender">the object which sends the event</param>
		/// <param name="e">the event handler args</param>
		private void ModifyRevenue_Click(object sender,EventArgs e)
		{
			if (CheckSecurityPermission(ModifyRevenue))
				MessageBox.Show(this,DataLayer.GetString(HasPermission),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Information);
			else
				MessageBox.Show(this,DataLayer.GetString(HasNotPermission),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Warning);
		}

		/// <summary>
		/// checks whether the user has modify profit rights
		/// </summary>
		/// <param name="sender">the object which sends the event</param>
		/// <param name="e">the event handler args</param>
		private void ModifyProfit_Click(object sender,EventArgs e)
		{
			if (CheckSecurityPermission(ModifyProfit))
				MessageBox.Show(this,DataLayer.GetString(HasPermission),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Information);
			else
				MessageBox.Show(this,DataLayer.GetString(HasNotPermission),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Warning);
		}

		/// <summary>
		/// checks whether the user has the permission or not; this does not walk the stack
		/// as the standard .NET security classes; so it is not complete secure according to 
		/// the .NET framework, but it is an easy way how to check application specific 
		/// permissions
		/// </summary>
		/// <param name="Permission">the permission to check for</param>
		/// <returns>returns true if the user has the permission otherwise false</returns>
		private bool CheckSecurityPermission(string Permission)
		{
			// if the current IPrincipal is of the same type as our custom secuirty principal then 
			// go and check the security right
			if (Thread.CurrentPrincipal is SecurityPrincipal)
			{
				SecurityPrincipal Principal = (SecurityPrincipal)Thread.CurrentPrincipal;

				// returns whether the user has the permission or not
				return Principal.HasPermission(Permission);
			}

			// if we have a standard IPrincipal in use then we can not check the permission
			// and we always return false
			else
				return false;
		}

		/// <summary>
		/// obtains the IPrincipal.IIdentiy and then checks whether it is from type UserIdentity; if it
		/// is we can obtain a wealth of user information
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void UserDetails_Click(object sender,EventArgs e)
		{
			// check whether the current IPrincipal.IIdentity is of our custom type
			if (Thread.CurrentPrincipal.Identity is UserIdentity)
			{
				// get a reference to this custom IIdentity type
				UserIdentity User = (UserIdentity)Thread.CurrentPrincipal.Identity;

				// get a list of all supported property names
				string[] PropertyNames = User.GetPropertyNames();
				string PropertyList = "\n";

				// now create a list of all properties plus its values
				for (int Count = 0; Count < PropertyNames.Length; Count++)
					PropertyList += "\n" + PropertyNames[Count] + " = " + User.GetProperty(PropertyNames[Count]);

				// now show the list with some contextual explanation
				MessageBox.Show(this,String.Format(DataLayer.GetString(CustomUserInfo),PropertyList,CultureInfo.InvariantCulture),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Information);
			}

			// otherwise show the property information of the standard IIdentity object with some contextual explanation
			else
				MessageBox.Show(this,String.Format(DataLayer.GetString(StandardUserInfo),"\n\n" + DataLayer.GetString(StandardPropertyName) + " = " + Thread.CurrentPrincipal.Identity.Name,CultureInfo.InvariantCulture),DataLayer.GetString(MessageCaption),MessageBoxButtons.OK,MessageBoxIcon.Information);
		}

		/// <summary>
		/// logs off the user by restoreing the original IPrincipal and restoring the windows form 
		/// to its original layout
		/// </summary>
		/// <param name="sender"></param>
		/// <param name="e"></param>
		private void LogOff_Click(object sender,EventArgs e)
		{
			// array with all the controls we created dynamically after the logon; any control
			// we find with these names will be removed from the Controls collection which results
			// in it being removed from the form and being destroyed
			string[] ControlNames = new string[20] { ViewBudget, ViewInventory, ViewRevenue, ViewProfit, 
													 UserRole, PowerUserRole, AdministratorRole, SuperAdministratorRole,
													 ModifyBudget, ModifyInventory, ModifyRevenue, ModifyProfit, 
													 HasNotPermission, HasPermission, LogOff, Cancel, UserDetails, 
													 SecurityGroupListName , SecurityRightListName, UserInfoListName };

			// now go through all the controls we identified to be removed
			for (int Count=0; Count < ControlNames.Length; Count++)
				
				// loop through all controls in the control collection of the window
				foreach (Control TheControl in this.Controls)

					// make sure we have a valid control 
					if (TheControl != null)

						// if the control name is the list of controls to be removed, remove it
						if (TheControl.Name == ControlNames[Count])
							this.Controls.Remove(TheControl);

			// show all original controls; used to perform a logon
			labelUserName.Visible = true;
			labelPassword.Visible = true;
			textUserName.Visible = true;
			textPassword.Visible = true;
			btnLogOn.Visible = true;

			// resize the form to its original size
			this.Size = new Size(264,152);

			// restores the original security principal
			SecurityPrincipal.RestoreSecurityPrincipal(OriginalPrincipal);
		}
	
		/// <summary>
		/// creates a new button and places it on the form; associates a click event handler and returns
		/// a reference of the button to the caller for further manipulation
		/// </summary>
		/// <param name="ButtonLocation">the location of the button on the form</param>
		/// <param name="Name">the name of the button</param>
		/// <param name="TextResourceName">the string resource name to be used to retrieve the button text</param>
		/// <param name="e">the event handler associated with the button click event</param>
		/// <returns>returns the button reference to the caller</returns>
		private Button AddButtonWithEventHandler(Point ButtonLocation,string Name,string TextResourceName,EventHandler e)
		{
			// create the button and set its location, standard size and name & text
			Button TheButton = new Button();
			TheButton.Location = ButtonLocation;
			TheButton.Size = new Size(100, 25);
			TheButton.Text = DataLayer.GetString(TextResourceName);
			TheButton.Name = Name;

			// add the button to the control list of the form
			this.Controls.Add(TheButton);

			// add the event handler to the button
			TheButton.Click += e;

			// show the button on the form
			TheButton.Show();

			// return the button to the caller so it can be further manipulated
			return TheButton;
		}

		/// <summary>
		/// create a list box and add all the items from the hash-table we pass on
		/// </summary>
		/// <param name="ListBoxLocation">the location on the form</param>
		/// <param name="ListBoxSize">the size of the list box</param>
		/// <param name="Name">the name of the list box</param>
		/// <param name="NameValueHashtable">a hash-table with values to add to the list box</param>
		/// <returns>returns the list box to the caller for furthe manipulation</returns>
		private ListBox AddListBoxWithValues(Point ListBoxLocation, Size ListBoxSize, string Name, Hashtable NameValueHashtable)
		{
			// create the list box and set its location, size and name
			ListBox TheListBox = new ListBox();
			TheListBox.Location = ListBoxLocation;
			TheListBox.Size = ListBoxSize;
			TheListBox.Name = Name;

			// add it to the list of controls of the form
			this.Controls.Add(TheListBox);

			// now read all values from the hash-table and add them to the lsit-box
			foreach(object Key in NameValueHashtable.Keys)
				TheListBox.Items.Add(Key + " = " + NameValueHashtable[Key]);

			// show the list box
			TheListBox.Show();

			// return the list box so it can be further manipulated
			return TheListBox;
		}
	}
}
